import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
import echarts from 'echarts';
import _Reflect$deleteProperty from 'babel-runtime/core-js/reflect/delete-property';
import _Reflect$apply from 'babel-runtime/core-js/reflect/apply';
import _Reflect$get from 'babel-runtime/core-js/reflect/get';
import _Reflect$set from 'babel-runtime/core-js/reflect/set';
import _createClass from 'babel-runtime/helpers/createClass';

/**
 * @description - helper method
 * @author - huang.jian <hjj491229492@hotmail.com>
 */

/**
 * @description - determine function variable
 *
 * @param {*} anything
 *
 * @return {boolean}
 */
function isFunction(anything) {
  return Object.prototype.toString.apply(anything) === '[object Function]';
}

/**
 * @description - generate unique identity for easier watch
 *
 * @return {string}
 */
function uuid() {
  return Math.random().toString(36).substr(2, 9);
}

/**
 * @description - proxy echarts instance, make none-element operation available
 * @author - huang.jian <hjj491229492@hotmail.com>
 */

/* eslint-disable no-unused-expressions */
var Proxy = function () {
  function Proxy() {
    _classCallCheck(this, Proxy);

    // middleware echarts instance
    this.instance = {};
    // flag stand for element initialize
    this.connected = false;
    // stock history options for controller cache or ng HMR
    this.history = {};
    // buffer event listener, stock history event listen for re-connect
    this.bufferEventListen = [];
    this.restoreBufferVariable();
    this.lazyMethodList = ['getWidth', 'getHeight', 'getDom', 'getOption', 'getDataURL', 'getConnectedDataURL', 'convertToPixel', 'convertFromPixel', 'containPixel', 'isDisposed', 'dispatchAction'];
  }

  /**
   * @description - 缓冲分组标记
   *
   * @see - http://echarts.baidu.com/api.html#echartsInstance.group
   */


  /**
   * @description - 统一管控缓存数据
   */
  Proxy.prototype.restoreBufferVariable = function restoreBufferVariable() {
    // buffer instance group setting
    this.bufferGroupCategory = '';
    // buffer DOM width and height setting
    this.bufferVisionSize = [];
    // buffer loading status switcher
    this.bufferLoadingSwitchery = [];
    // buffer options setting
    this.bufferOptions = [];
  };

  /**
   * @description - 传递echarts实例
   *
   * @see - http://echarts.baidu.com/api.html#echartsInstance
   *
   * @param {object} instance
   */


  Proxy.prototype.transferCoreBridge = function transferCoreBridge(instance) {
    var _this = this;

    this.instance = instance;
    this.connected = true;

    // 延后加载非关键部分方法，属性
    this.lazyMethodList.forEach(function (key) {
      _Reflect$set(_this, key, _Reflect$get(_this.instance, key).bind(_this.instance));
    });
  };

  /**
   * @description - 消费实例化前缓存数据
   */


  Proxy.prototype.dealWithInventory = function dealWithInventory() {
    var _this2 = this;

    this.group = this.bufferGroupCategory;
    this.bufferVisionSize.length && this.resize.apply(this, this.bufferVisionSize);
    this.bufferLoadingSwitchery.length && this.showLoading.apply(this, this.bufferLoadingSwitchery);
    this.bufferOptions.length && this.bufferOptions.forEach(function (args) {
      return _this2.setOption.apply(_this2, args);
    });
    // eslint-disable-next-line max-len
    this.bufferEventListen.length && this.bufferEventListen.forEach(function (stock) {
      return _this2.on(stock.eventName, stock.handler);
    });

    this.restoreBufferVariable();
  };

  /**
   * @description - 缓冲setOption调用
   *
   * @see - http://echarts.baidu.com/api.html#echartsInstance.setOption
   */


  Proxy.prototype.setOption = function setOption() {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    // eslint-disable-next-line max-len
    this.connected ? _Reflect$apply(this.instance.setOption, this.instance, args) : this.bufferOptions.push(args);
    this.connected && (this.history = _Reflect$apply(this.instance.getOption, this.instance, []));
    return this;
  };

  /**
   * @description - 式缓冲resize调用
   *
   * @see - http://echarts.baidu.com/api.html#echartsInstance.on
   */


  Proxy.prototype.resize = function resize() {
    for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    // eslint-disable-next-line max-len
    this.connected ? _Reflect$apply(this.instance.resize, this.instance, args) : this.bufferVisionSize = args;
    return this;
  };

  /**
   * @description - 缓冲事件绑定
   *
   * @see - http://echarts.baidu.com/api.html#echartsInstance.on
   *
   * @param {string} eventName
   * @param {function} handler
   *
   * @return {MonkeyPatchBridge}
   */


  Proxy.prototype.on = function on(eventName, handler) {
    if (this.connected) {
      _Reflect$apply(this.instance.on, this.instance, [eventName, handler]);
    } else {
      this.bufferEventListen = [].concat(this.bufferEventListen, [{ eventName: eventName, handler: handler }]);
    }

    return this;
  };

  /**
   * @description - 缓冲事件解绑
   *
   * @see - http://echarts.baidu.com/api.html#echartsInstance.off
   *
   * @param {string} eventName
   * @param {function} handler
   *
   * @return {MonkeyPatchBridge}
   */


  Proxy.prototype.off = function off(eventName, handler) {
    if (this.connected) {
      _Reflect$apply(this.instance.off, this.instance, [eventName, handler]);
    } else {
      // eslint-disable-next-line arrow-body-style
      this.bufferEventListen = this.bufferEventListen.filter(function (stock) {
        return isFunction(handler) ? !(stock.eventName === eventName && stock.handler === handler) : !(stock.eventName === eventName);
      });
    }

    return this;
  };

  /**
   * @description - 处理controller实例重连接引发的潜在问题
   */


  Proxy.prototype.replay = function replay() {
    this.instance.setOption(this.history);
    return this;
  };

  /**
   * @description - 缓冲showLoading调用
   *
   * @see - http://echarts.baidu.com/api.html#echartsInstance.showLoading
   */


  Proxy.prototype.showLoading = function showLoading() {
    for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      args[_key3] = arguments[_key3];
    }

    // eslint-disable-next-line max-len
    this.connected ? _Reflect$apply(this.instance.showLoading, this.instance, args) : this.bufferLoadingSwitchery = args.length ? args : ['default'];
    return this;
  };

  /**
   * @description - 缓冲hideLoading调用
   *
   * @see - http://echarts.baidu.com/api.html#echartsInstance.hideLoading
   */


  Proxy.prototype.hideLoading = function hideLoading() {
    // eslint-disable-next-line max-len
    this.connected ? _Reflect$apply(this.instance.hideLoading, this.instance, []) : this.bufferLoadingSwitchery = [];
    return this;
  };

  /**
   * @description - 缓冲clear调用
   *
   * @see - http://echarts.baidu.com/api.html#echartsInstance.clear
   */


  Proxy.prototype.clear = function clear() {
    this.connected && this.instance.clear();
    return this;
  };

  /**
   * @description - 缓冲dispose调用
   *
   * @see - http://echarts.baidu.com/api.html#echartsInstance.dispose
   */


  Proxy.prototype.dispose = function dispose() {
    var _this3 = this;

    this.connected && this.instance.dispose();
    this.instance = {};
    this.connected = false;
    // reset buffer content
    this.restoreBufferVariable();
    // 剔除延迟加载方法
    this.lazyMethodList.forEach(function (key) {
      _Reflect$deleteProperty(_this3, key);
    });
  };

  _createClass(Proxy, [{
    key: 'group',
    get: function get() {
      return this.connected ? _Reflect$get(this.instance, 'group') : this.bufferGroupCategory;
    },
    set: function set(value) {
      this.connected ? _Reflect$set(this.instance, 'group', value) : this.bufferGroupCategory = value;
      return value;
    }
  }]);

  return Proxy;
}();

/**
 * @description - echarts bridge to cache operation
 * @author - huang.jian <hjj491229492@hotmail.com>
 */

var Bridge = function (_Proxy) {
  _inherits(Bridge, _Proxy);

  /**
   * @description - echarts bridge instance
   *
   * @param {string} theme - echarts theme
   * @param {object} initOptions - echarts init options
   * @param {object} mediaOptions - echarts media options
   */
  function Bridge(theme) {
    var initOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var mediaOptions = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

    _classCallCheck(this, Bridge);

    var _this = _possibleConstructorReturn(this, _Proxy.call(this));

    _this.theme = theme;
    _this.initOptions = initOptions;
    _this.mediaOptions = mediaOptions;
    _this.uuid = uuid();
    return _this;
  }

  /**
   * @description - 代理echarts初始化函数，只需要传入关键element即可
   *
   * @param {HTMLElement} element
   */


  Bridge.prototype.connect = function connect(element) {
    if (!(element instanceof HTMLElement)) {
      throw new Error('HTMLElement argument required');
    }

    // 此处为部分hack操作
    // 限定媒体查询，简易声明对象时直接指定，而不是后续控制
    this.middleware = echarts.init(element, this.theme, this.initOptions);
    this.middleware.setOption({ series: [] });
    this.middleware.setOption({ media: this.mediaOptions });
    // 传递实例，消费缓存数据
    _Proxy.prototype.transferCoreBridge.call(this, this.middleware);
    _Proxy.prototype.dealWithInventory.call(this);

    return this;
  };

  /**
   * @description - 销毁应用实例
   */


  Bridge.prototype.disconnect = function disconnect() {
    _Proxy.prototype.clear.call(this);
    _Proxy.prototype.dispose.call(this);
  };

  return Bridge;
}(Proxy);

/**
 * @description - nothing but rollup bundle
 * @author - huang.jian <hjj491229492@hotmail.com>
 */
// eslint-disable-next-line import/prefer-default-export

export { Bridge };
